from flask import Flask
from prometheus_client import Counter, Gauge, start_http_server
from prometheus_client import generate_latest, CONTENT_TYPE_LATEST, CollectorRegistry
import psutil
import time

app = Flask(__name__)

# Define Prometheus metrics
function_requests_total = Counter('function_requests_total', 'Total number of requests to the function')
cpu_utilization_gauge = Gauge('cpu_utilization_percentage', 'CPU utilization percentage')
request_rate_gauge = Gauge('function_request_rate', 'Function request rate (requests per second)')
service_time_gauge = Gauge('function_service_time_seconds', 'Service time of the function in seconds')
network_bytes_sent = Counter('network_bytes_sent_total', 'Total number of bytes sent over the network')
network_bytes_recv = Counter('network_bytes_received_total', 'Total number of bytes received over the network')


start_time=time.time()

@app.route("/clear_metrics")
def clear_metrics():
    function_requests_total._value.set(0)
    cpu_utilization_gauge.set(0)
    service_time_gauge.set(0)
    network_bytes_sent._value.set(0)
    network_bytes_recv._value.set(0)
    start_time=time.time()
    registry = CollectorRegistry()
    return generate_latest(registry), 200, {"Content-Type": CONTENT_TYPE_LATEST}


@app.route("/")
def hello():
    time_invoked=time.time()

    # Increment request counter
    function_requests_total.inc()

    time.sleep(0.1)
    

    service_time = time.time() - time_invoked

    service_time_gauge.set(service_time)
    
    # Collect CPU utilization
    cpu_utilization = psutil.cpu_percent()
    cpu_utilization_gauge.set(cpu_utilization)


    return "Hello from Python function"

@app.route("/metrics")
def metrics():
    registry = CollectorRegistry()
    # Register your Prometheus metrics collectors here
    # For example:
    # registry.register(...)

    elapsed_time = time.time() - start_time

    registry.register(function_requests_total)
    registry.register(cpu_utilization_gauge)
    registry.register(service_time_gauge)
    registry.register(network_bytes_sent)
    registry.register(network_bytes_recv)
    

    metric=generate_latest(registry)
    for line in metric.decode().split("\n"):
        if "function_requests_total" in line:
            try:
                value = float(line.split()[1])
                request_rate = value/elapsed_time
            except:
                pass
        if "function_requests_total" in line:
            try:
                no_of_requests = float(line.split()[1])
            except:
                pass
        if "cpu_utilization_percentage" in line:
            try:
                cpu_utilization = float(line.split()[1])
            except:
                pass
        if "function_service_time_seconds" in line:
            try:
                service_time= float(line.split()[1])
            except:
                pass

        if "network_bytes_sent_total" in line:
            try:
                bytes_sent= float(line.split()[1])
            except:
                pass
        if "network_bytes_received_total" in line:
            try:
                bytes_received= float(line.split()[1])
            except:
                pass
    request_rate_gauge.set(request_rate)
    registry.register(request_rate_gauge)


    network_stats = psutil.net_io_counters()
    network_bytes_sent.inc(network_stats.bytes_sent)
    network_bytes_recv.inc(network_stats.bytes_recv)


    print("Number of Requests: ",no_of_requests)
    print("Request_rate: ",request_rate)
    print("Service Time: ",service_time)
    print("CPU utilization: ",cpu_utilization)
    print("Bytes send: ",bytes_sent)
    print("Bytes received: ",bytes_received)

    return generate_latest(registry), 200, {"Content-Type": CONTENT_TYPE_LATEST}


if __name__ == "__main__":
    # Start Prometheus HTTP server to expose metrics
    start_http_server(8000)
    
    # Run Flask app
    app.run(host='0.0.0.0')
